@@ -11,15 +11,10 @@
 #   Cluster
 
 package Prima::Buttons;
-
-use Carp;
-use Prima::Const;
-use Prima::Classes;
-use Prima::IntUtils;
-use Prima::StdBitmap;
 use strict;
 use warnings;
-
+use Carp;
+use Prima qw(IntUtils StdBitmap);
 
 package Prima::AbstractButton;
 use vars qw(@ISA);
@@ -266,47 +261,33 @@ sub draw_veil
 sub draw_caption
 {
 	my ( $self, $canvas, $x, $y) = @_;
-	my $cap = $self-> text;
-	my ( $leftPart, $accel);
-	unless ( ref $cap ) {
-		$cap =~ s/^([^~]*)\~(.*)$/$1$2/;
-		( $leftPart, $accel) = ( $1,
-			( defined ($2) && length($2)) ? substr( $2, 0, 1) : undef);
-	}
-	my ( $fw, $fh, $enabled) = (
-		$canvas-> get_text_width( $cap),
-		$canvas-> font-> height,
-		$self-> enabled
-	);
-
-	if ( defined $accel)
-	{
-		my ( $a, $b, $c) = (
-			$canvas-> get_text_width( $leftPart),
-			$canvas-> get_text_width( $leftPart.$accel),
-			$canvas-> get_text_width( $accel)
-		);
-		unless ( $enabled)
-		{
-			my $z = $canvas-> color;
-			$canvas-> color( cl::White);
-			$canvas-> line( $x + $b - $c + 1, $y - 1, $x + $b * 2 - $a - $c, $y - 1);
-			$canvas-> color( $z);
-		}
-		$canvas-> line( $x + $b - $c, $y, $x + $b * 2 - $a - $c - 1, $y);
-	}
-
-	unless ( $enabled)
-	{
-		my $c = $canvas-> color;
+	my ($cap, $tilde) = @{ $self-> text_wrap_shape( $self-> text, 
+		undef, 
+		options => tw::CalcMnemonic|tw::CollapseTilde|tw::ExpandTabs,
+		tabs    => 1,
+	) };
+	unless ( $self->enabled) {
+		my $z = $canvas-> color;
 		$canvas-> color( cl::White);
-		$canvas-> text_out_bidi( $cap, $x+1, $y-1);
-		$canvas-> color( $c);
+		$canvas-> text_out( $cap, $x + 1, $y - 1);
+		$canvas->line( 
+			$x + 1 + $tilde->{tildeStart}, $y - 1, 
+			$x + 1 + $tilde->{tildeEnd}, $y - 1, 
+		) if defined $tilde->{tildeLine};
+		$canvas-> color( $z);
+	}
+	$canvas-> text_out( $cap, $x + 1, $y - 1);
+	$canvas->line( 
+		$x + $tilde->{tildeStart}, $y, 
+		$x + $tilde->{tildeEnd}, $y, 
+	) if defined $tilde->{tildeLine};
+	if ($self-> focused) {
+		my ( $fw, $fh) = (
+			$canvas-> get_text_width( $cap),
+			$canvas-> font-> height,
+		);
+		$canvas-> rect_focus( $x - 2, $y - 2, $x + 2 + $fw, $y + 2 + $fh)
 	}
-
-	$canvas-> text_out_bidi( $cap, $x, $y);
-	$canvas-> rect_focus( $x - 2, $y - 2, $x + 2 + $fw, $y + 2 + $fh)
-		if $self-> focused;
 }
 
 sub caption_box
@@ -327,18 +308,15 @@ sub pressed
 	$_[0]-> repaint;
 }
 
-
-sub text
+sub set_text
 {
-	return $_[0]-> SUPER::text unless $#_;
 	my ( $self, $caption) = @_;
-	$self-> SUPER::text( $caption );
+	$self-> SUPER::set_text( $caption );
 	$self-> {accel} = lc($1) if $caption =~ /~([a-z0-9])/i;
 	$self-> check_auto_size;
 	$self-> repaint;
 }
 
-
 sub on_enable  { $_[0]-> repaint; }
 sub on_disable { $_[0]-> cancel_transaction; $_[0]-> repaint; }
 sub on_enter   { $_[0]-> repaint; }
@@ -494,25 +472,44 @@ sub on_paint
 		my $sh = $ph * $is;
 		my $imgNo = $self-> {defaultGlyph};
 		my $useVeil = 0;
+		my $image = $self->{image};
 		if ( $self-> {hilite}) {
-			$imgNo = $self-> {hiliteGlyph}
-				if $self-> {glyphs} > $self-> {hiliteGlyph} &&
-					$self-> {hiliteGlyph} >= 0;
+			if ( $self->{glyphs} > 1 ) {
+				$imgNo = $self-> {hiliteGlyph}
+					if $self-> {glyphs} > $self-> {hiliteGlyph} &&
+						$self-> {hiliteGlyph} >= 0;
+			} elsif ( ref($self-> {hiliteGlyph})) {
+				$image = $self->{hiliteGlyph};
+			}
 		}
 		if ( $self-> {checked}) {
-			$imgNo = $self-> {holdGlyph} if
-				$self-> {glyphs} > $self-> {holdGlyph} &&
-					$self-> {holdGlyph} >= 0;
+			if ( $self->{glyphs} > 1 ) {
+				$imgNo = $self-> {holdGlyph} if
+					$self-> {glyphs} > $self-> {holdGlyph} &&
+						$self-> {holdGlyph} >= 0;
+			} elsif ( ref($self->{holdGlyph})) {
+				$image = $self->{holdGlyph};
+			}
 		}
 		if ( $self-> {pressed}) {
-			$imgNo = $self-> {pressedGlyph} if
-				$self-> {glyphs} > $self-> {pressedGlyph} &&
-					$self-> {pressedGlyph} >= 0;
+			if ( $self->{glyphs} > 1 ) {
+				$imgNo = $self-> {pressedGlyph} if
+					$self-> {glyphs} > $self-> {pressedGlyph} &&
+						$self-> {pressedGlyph} >= 0;
+			} elsif ( ref($self->{pressedGlyph}) ) {
+				$image = $self->{pressedGlyph};
+			}
 		}
 		if ( !$self-> enabled) {
-			( $self-> {glyphs} > $self-> {disabledGlyph} && $self-> {disabledGlyph} >= 0) ?
-				$imgNo = $self-> {disabledGlyph} :
-					$useVeil = 1;
+			if ( $self->{glyphs} > 1 ) {
+				( $self-> {glyphs} > $self-> {disabledGlyph} && $self-> {disabledGlyph} >= 0) ?
+					$imgNo = $self-> {disabledGlyph} :
+						$useVeil = 1;
+			} elsif (ref($self->{disabledGlyph})) {
+				$image = $self->{disabledGlyph};
+			} else {
+				$useVeil = 1;
+			}
 		}
 
 		my ( $imAtX, $imAtY);
@@ -535,7 +532,16 @@ sub on_paint
 			$imAtY = ( $size[1] - $sh) / 2 - $shift;
 		}
 
-		my $image = $self->{image};
+		if ( $image && UNIVERSAL::isa($image, 'Prima::Drawable::Metafile')) {
+			if ( !$self->enabled && $useVeil && $image->type == dbt::Bitmap) {
+				$canvas->color(cl::White);
+				$image->execute($canvas, $imAtX+1, $imAtY-1);
+				$useVeil = 0;
+			}
+			$canvas->color($clr[0]);
+			$image->execute($canvas, $imAtX, $imAtY);
+			goto CAPTION;
+		}
 		if ( $self-> {smoothScaling} && $is != 1.0 ) {
 			my $c = $self->{smooth_cache} //= {
 				zoom  => -1,
@@ -562,6 +568,7 @@ sub on_paint
 			$pw, $ph,
 			rop::CopyPut
 		);
+CAPTION:
 		$self-> draw_veil( $canvas, $imAtX, $imAtY, $imAtX + $sw, $imAtY + $sh)
 			if $useVeil;
 	} else {
@@ -1162,15 +1169,16 @@ sub on_paint
 	$canvas-> rectangle( 1, 0, $size[0] - 1, $size[1] - $fh / 2 - 2);
 	$canvas-> color( $self-> dark3DColor);
 	$canvas-> rectangle( 0, 1, $size[0] - 2, $size[1] - $fh / 2 - 1);
-	my $c = $self-> text;
+	my $c = $self->text;
 	if ( length( $c) > 0) {
+		$c = $self-> text_shape($c, skip_if_simple => 1) || $c;
 		$canvas-> color( $clr[1]);
 		$canvas-> bar  (
 			8, $size[1] - $fh - 1,
 			16 + $canvas-> get_text_width( $c), $size[1] - 1
 		);
 		$canvas-> color( $clr[0]);
-		$canvas-> text_out_bidi( $c, 12, $size[1] - $fh - 1);
+		$canvas-> text_out( $c, 12, $size[1] - $fh - 1);
 	}
 }
 
@@ -1246,7 +1254,7 @@ Prima::Buttons - button widgets and grouping widgets.
 
 	run Prima;
 
-=for podview <img src="buttons.gif" cut=1>
+=for podview <img src="buttons.gif">
 
 =for html <p><img src="https://raw.githubusercontent.com/dk/Prima/master/pod/Prima/buttons.gif">
 
@@ -1442,13 +1450,13 @@ the 'default' action. Useful for OK-buttons in dialogs.
 
 Default value: 0
 
-=item defaultGlyph INTEGER
+=item defaultGlyph INTEGER | IMAGE | METAFILE
 
 Selects index of the default sub-image.
 
 Default value: 0
 
-=item disabledGlyph INTEGER
+=item disabledGlyph INTEGER | IMAGE | METAFILE
 
 Selects index of the sub-image for the disabled button state.
 If C<image> does not contain such sub-image, the C<defaultGlyph>
@@ -1479,7 +1487,7 @@ L<hiliteGlyph>, L<disabledGlyph>, L<pressedGlyph>, L<holdGlyph>.
 
 Default value: 1
 
-=item hiliteGlyph INTEGER
+=item hiliteGlyph INTEGE | IMAGE | METAFILER
 
 Selects index of the sub-image for the state when the mouse pointer is
 over the button. This image is used only when L<flat> property is set.
@@ -1487,7 +1495,7 @@ If C<image> does not contain such sub-image, the C<defaultGlyph> sub-image is dr
 
 Default value: 0
 
-=item holdGlyph INTEGER
+=item holdGlyph INTEGE | IMAGE | METAFILER
 
 Selects index of the sub-image for the state when the button is L<checked>.
 This image is used only when L<checkable> property is set.
@@ -1501,6 +1509,9 @@ If set, the image object is drawn next with the button text, over or left to it
 ( see L<vertical> property ). If OBJECT contains several sub-images, then the
 corresponding sub-image is drawn for each button state. See L<glyphs> property.
 
+Can also be a C<Prima::Drawable::Metafile> object, however, C<imageScale> factor
+wouldn't work on it.
+
 Default value: undef
 
 =item imageFile FILENAME
@@ -1559,7 +1570,7 @@ Default value: true
 
 See also: L<Prima::Image/ui_scale> .
 
-=item pressedGlyph INTEGER
+=item pressedGlyph INTEGER | IMAGE | METAFILE
 
 Selects index of the sub-image for the pressed state of the button.
 If C<image> does not contain such sub-image, the C<defaultGlyph> sub-image is drawn.
@@ -1724,6 +1735,7 @@ Dmitry Karasik, E<lt>dmitry@karasik.eu.orgE<gt>.
 =head1 SEE ALSO
 
 L<Prima>, L<Prima::Widget>, L<Prima::Window>, L<Prima::IntUtils>,
+L<Prima::Drawable::Metafile>,
 L<Prima::StdBitmap>, F<examples/buttons.pl>, F<examples/buttons2.pl>.
 
 =cut
