@@ -1,33 +1,5 @@
-# $Id$
-#
-#  Copyright (c) 1997-2004 Dmitry Karasik
-#  All rights reserved.
-#
-#  Redistribution and use in source and binary forms, with or without
-#  modification, are permitted provided that the following conditions
-#  are met:
-#  1. Redistributions of source code must retain the above copyright
-#     notice, this list of conditions and the following disclaimer.
-#  2. Redistributions in binary form must reproduce the above copyright
-#     notice, this list of conditions and the following disclaimer in the
-#     documentation and/or other materials provided with the distribution.
-#
-#  THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
-#  ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
-#  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
-#  ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
-#  FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
-#  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
-#  OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
-#  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
-#  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
-#  OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
-#  SUCH DAMAGE.
-#
-#  Created by Dmitry Karasik <dmitry@karasik.eu.org>
-#
-#  $Id$
 use strict;
+use warnings;
 
 package Prima::EventHook;
 use vars qw($hook $auto_hook %hooks %groups);
@@ -48,12 +20,12 @@ $auto_hook = 1;
 sub install
 {
 	my ( $sub, %rules) = @_;
-	
+
 	my @params;
 	if ( defined($rules{param})) {
 		@params = ( ref($rules{param}) eq 'ARRAY') ? @{$rules{param}} : $rules{param};
 	}
-	
+
 	my @names;
 	if ( defined($rules{event})) {
 		@names = ( ref($rules{event}) eq 'ARRAY') ? @{$rules{event}} : $rules{event};
@@ -68,7 +40,7 @@ sub install
 	} else {
 		@objects = (undef);
 	}
-	
+
 	for (@names) {
 		$hooks{$_} = [] unless $hooks{$_};
 		my $array = $hooks{$_};
@@ -77,7 +49,7 @@ sub install
 		}
 	}
 
-	Prima::Component-> event_hook( $hook = \&_hook_proc) 
+	Prima::Component-> event_hook( $hook = \&_hook_proc)
 		if $auto_hook && !$hook;
 }
 
@@ -89,7 +61,7 @@ sub deinstall
 		@{$hooks{$_}} = grep { $$_[0] != $sub } @{$hooks{$_}};
 		$total += @{$hooks{$_}};
 	}
-	Prima::Component-> event_hook( $hook = undef) 
+	Prima::Component-> event_hook( $hook = undef)
 		if !$total && $hook && $auto_hook;
 }
 
@@ -100,7 +72,7 @@ sub _hook_proc
 		next unless exists $hooks{$_};
 		for ( @{$hooks{$_}}) {
 			my ( $sub, $sub_object, $sub_children, @sub_params) = @$_;
-			next if 
+			next if
 				defined $sub_object &&
 				(
 					(  $sub_children && $sub_object-> is_owner( $object) == 0) ||
@@ -114,8 +86,6 @@ sub _hook_proc
 
 1;
 
-__DATA__
-
 =pod
 
 =head1 NAME
@@ -135,9 +105,9 @@ Prima::EventHook - event filtering
 		return 1;
 	}
 
-	Prima::EventHook::install( \&hook, 
+	Prima::EventHook::install( \&hook,
 		param    => $my_param,
-		object   => $my_window, 
+		object   => $my_window,
 		event    => [qw(Size Move Destroy)],
 		children => 1
 	);
@@ -147,10 +117,10 @@ Prima::EventHook - event filtering
 =head1 DESCRIPTION
 
 Prima dispatches events by calling notifications registered
-on one or more objects interested in the events. Also, one 
+on one or more objects interested in the events. Also, one
 event hook can be installed that would receive all events occurred on
 all objects. C<Prima::EventHook> provides multiplex access to
-the core event hook and introduces set of dispatching rules so 
+the core event hook and introduces set of dispatching rules so
 the user hook subs receive only a defined subset of events.
 
 The filtering criteria are event names and object hierarchy.
@@ -176,7 +146,7 @@ Rules can contain the following keys:
 
 Event is either a string, an array of strings, or C<undef> value.  In the latter
 case it is equal to C<'*'> string, which selects all events to be passed in the
-SUB. A string is either name of an event, or one of pre-defined event groups, 
+SUB. A string is either name of an event, or one of pre-defined event groups,
 declared in C<%groups> package hash. The group names are:
 
 	ability
@@ -184,7 +154,7 @@ declared in C<%groups> package hash. The group names are:
 	geometry
 	keyboard
 	menu
-	mouse  
+	mouse
 	objects
 	visibility
 
@@ -213,7 +183,7 @@ Setting together object to C<undef> and children to 1 is inefficient.
 
 =item param
 
-A scalar or array of scalars passed as first parameters to SUB 
+A scalar or array of scalars passed as first parameters to SUB
 whenever it is called.
 
 =back
