@@ -22,7 +22,7 @@ sub profile_default
 		autoHeight     => 1,
 		autoSelect     => 1,
 		autoTab        => 0,
-		borderWidth    => 1,
+		borderWidth    => 2,
 		charOffset     => 0,
 		cursorVisible  => 1,
 		cursorSize     => [ Prima::Application-> get_default_cursor_width, $font-> { height}],
@@ -43,6 +43,7 @@ sub profile_default
 			[undo        => '~Undo', 'Ctrl+Z', '^Z', 'undo'],
 			[redo        => 'R~edo', 'Ctrl+Y', '^Y', 'redo'],
 			['@rtl'      => '~RTL input', 'Ctrl+Shift+D', '^#D', 'toggle_rtl'],
+			['@ligation' => '~Ligation', 'Ctrl+Shift+L', '^#L', 'toggle_ligation'],
 		],
 		readOnly       => 0,
 		selection      => [0, 0],
@@ -50,6 +51,7 @@ sub profile_default
 		selEnd         => 0,
 		selectable     => 1,
 		textDirection  => $rtl,
+		textLigation   => 1,
 		undoLimit      => 10,
 		widgetClass    => wc::InputLine,
 		width          => 96,
@@ -75,7 +77,7 @@ sub init
 
 	for ( qw(
 		borderWidth passwordChar maxLen alignment autoTab autoSelect
-		firstChar charOffset readOnly))
+		firstChar charOffset readOnly textLigation))
 		{ $self-> {$_} = 1; }
 	for ( qw( selStart selEnd atDrawX autoHeight undoLimit n_clusters))
 		{ $self-> {$_} = 0;}
@@ -89,7 +91,7 @@ sub init
 	$self->init_undo(\%profile);
 
 	for ( qw(
-		textDirection
+		textDirection textLigation
 		writeOnly borderWidth passwordChar maxLen alignment
 		autoTab autoSelect readOnly selEnd selStart charOffset
 		firstChar wordDelimiters ))
@@ -114,17 +116,20 @@ sub on_paint
 	my @clr;
 	my @selClr;
 	@clr = $self-> enabled ?
-		($self-> color, $self-> backColor) :
+		($self-> color,
+			$self->{prelight} ?
+				$self-> prelight_color($self-> backColor) :
+				$self-> backColor
+		) :
 		($self-> disabledColor, $self-> disabledBackColor);
 	@selClr = ($self-> hiliteColor, $self-> hiliteBackColor);
 
 	my $border = $self-> {borderWidth};
-	if ( $self-> {borderWidth} == 0) {
-		$canvas-> color( $clr[1]);
-		$canvas-> bar(0,0,@size);
-	} else {
-		$canvas-> rect3d( 0, 0, $size[0]-1, $size[1]-1, $border, $self-> dark3DColor, $self-> light3DColor, $clr[1]);
-	}
+	$self-> rect_bevel( $canvas, Prima::rect->new(@size)->inclusive,
+		width  => $self-> {borderWidth},
+		panel  => 1,
+		fill   => $clr[1],
+	);
 
 	return if $size[0] <= $border * 2 + 2;
 	my $cap = $self-> {glyphs} or return;
@@ -147,11 +152,13 @@ sub on_paint
 
 	my ( $x, $y) = ( $self-> {atDrawX}, $self-> {atDrawY});
 	if ( $useSel && @{ $self->{selChunks} // [] }) {
-		$self->{glyphs}->selection_walk(
+		my $g = $self->{glyphs};
+		$g->selection_walk(
 			$self->{selChunks}, $self->{firstChar}, $self->{n_clusters}, sub {
 			my ( $offset, $length, $selected ) = @_;
-			my $text = substr( $cap, $offset, $length );
-			my $dx = $self->{glyphs}->get_sub_width( $self, $self->{firstChar} + $offset, $length);
+			($offset, $length) = $g->cluster2glyph($self->{firstChar} + $offset, $length);
+
+			my $dx = $canvas->get_text_width( $g, 0, $offset, $length);
 			if ( $selected ) {
 				$canvas-> color( $self-> hiliteBackColor);
 				$canvas-> bar( $x, 0, $x + $dx - 1, $size[1] - 1);
@@ -159,7 +166,7 @@ sub on_paint
 			} else {
 				$canvas-> color( $clr[0]);
 			}
-			$canvas->text_out($self->{glyphs}, $x, $y, $self->{firstChar} + $offset, $length );
+			$canvas->text_out($g, $x, $y, $offset, $length);
 			$x += $dx;
 		});
 	} else {
@@ -231,19 +238,22 @@ sub reset
 	$self-> cursorPos( $x, $border + 1);
 }
 
-sub text
+sub set_text
 {
-	return $_[0]-> SUPER::text unless $#_;
 	my ( $self, $cap) = @_;
 	$cap = '' unless defined $cap;
 	$cap = substr( $cap, 0, $self-> {maxLen})
 		if $self-> {maxLen} >= 0 and length($cap) > $self-> {maxLen};
 
-	$self-> SUPER::text($cap);
+	$self-> SUPER::set_text($cap);
 
 	$cap = $self-> {passwordChar} x length $cap if $self-> {writeOnly};
 	if ( length($cap)) {
-		$self->{glyphs}     = $self-> text_shape( $cap, rtl => $self->textDirection );
+		$self->{glyphs}     = $self-> text_shape( $cap, 
+			rtl      => $self->textDirection,
+			level    => ( $self->textLigation ? ts::Full : ts::Glyphs ),
+			advances => 1,
+		);
 		$self->{n_clusters} = $self->{glyphs}->n_clusters;
 	} else {
 		$self->{glyphs}     = Prima::Drawable::Glyphs->new_empty;
@@ -523,6 +533,7 @@ sub on_popup
 	$p-> enabled( 'undo',         $self->can_undo );
 	$p-> enabled( 'redo',         $self->can_redo );
 	$p-> checked( 'rtl',          $self-> textDirection );
+	$p-> checked( 'ligation',     $self-> textLigation );
 }
 
 sub default_geom_height
@@ -555,6 +566,12 @@ sub toggle_rtl
 	$self-> textDirection($value);
 }
 
+sub toggle_ligation
+{	
+	my ( $self, $menu, $value ) = @_;
+	$self-> textLigation($value);
+}
+
 sub paste
 {
 	my $self = $_[0];
@@ -781,6 +798,22 @@ sub on_mouseup
 	$cp-> text(substr( $self-> text, $start, $end - $start)) if $start != $end;
 }
 
+sub on_mouseenter
+{
+	my $self = shift;
+	$self->{prelight} = 1;
+	return unless $self->enabled;
+	$self->repaint;
+}
+
+sub on_mouseleave
+{
+	my $self = shift;
+	delete $self->{prelight};
+	return unless $self->enabled;
+	$self->repaint;
+}
+
 sub on_size
 {
 	my $self = $_[0];
@@ -1054,6 +1087,14 @@ sub autoHeight
 	$_[0]-> check_auto_size;
 }
 
+sub textLigation
+{
+	return $_[0]-> {textLigation} unless $#_;
+	my ( $self, $textLigation ) = @_;
+	$self-> {textLigation} = $textLigation;
+	$self-> text( $self-> text );
+}
+
 sub textDirection
 {
 	return $_[0]-> {textDirection} unless $#_;
@@ -1111,7 +1152,7 @@ Prima::InputLine - standard input line widget
 	Prima::InputLine-> new( text => 'Hello world!');
 	run Prima;
 
-=for podview <img src="inputline.gif" cut=1>
+=for podview <img src="inputline.gif">
 
 =for html <p><img src="https://raw.githubusercontent.com/dk/Prima/master/pod/Prima/inputline.gif">
 
@@ -1229,6 +1270,16 @@ Selects the end of text selection.
 
 If set, indicates RTL text input.
 
+=item textLigation BOOLEAN
+
+If set, text may be rendered at better quality with ligation and kerning,
+however that comes with a price that some ligatures may be indivisible and form
+clusters (f.ex. I<ff> or I<ffi> ligatures). Cursor cannot go inside of such
+clusters, and thus one can only select them, delete as whole, or press
+Del/Backspace on the cluster's edge.
+
+Toggle during runtime with Ctrl+Shift+L.
+
 =item wordDelimiters STRING
 
 Contains string of character that are used for locating a word break.
