@@ -7,23 +7,17 @@ use vars qw(@ISA @EXPORT @EXPORT_OK);
 @ISA = qw(Exporter);
 @EXPORT = qw(message_box message input_box);
 @EXPORT_OK = qw(message_box message input_box);
-
-use Prima::Classes;
-use Prima::Buttons;
-use Prima::StdBitmap;
-use Prima::Label;
-use Prima::InputLine;
-use Prima::Utils;
+use Prima qw(Buttons InputLine Label ScrollWidget StdBitmap Utils);
 
 sub insert_buttons
 {
 	my ( $dlg, $buttons, $extras) = @_;
 	my ( $left, $right) = ( 10, 425);
 	my $i;
-	my @bConsts =  ( 
+	my @bConsts =  (
 		mb::Help, mb::Cancel, mb::Ignore, mb::Retry, mb::Abort, mb::No, mb::Yes, mb::Ok
 	);
-	my @bTexts  = qw( 
+	my @bTexts  = qw(
 		~Help    ~Cancel     ~Ignore     ~Retry     ~Abort     ~No     ~Yes     ~OK
 	);
 	my $helpTopic = defined $$extras{helpTopic} ? $$extras{helpTopic} : 'Prima';
@@ -40,7 +34,7 @@ sub insert_buttons
 		next unless $buttons & $bConsts[$i];
 		my %epr;
 
-		%epr = %{$extras-> {buttons}-> {$bConsts[$i]}} 
+		%epr = %{$extras-> {buttons}-> {$bConsts[$i]}}
 			if $extras-> {buttons} && $extras-> {buttons}-> {$bConsts[$i]};
 
 		my %hpr = (
@@ -74,14 +68,19 @@ sub insert_buttons
 		last if ++$btns > 3;
 	}
 	$fresh = $freshFirst unless $dir;
-	return $fresh;   
+	return $fresh;
 }
 
 sub message_box
 {
 	my ( $title, $text, $options, @extras) = @_;
-	
-	my $extras = 
+
+	if ( (ref($text) // '') eq 'SCALAR') {
+		require Prima::Drawable::Markup;
+		$text = Prima::Drawable::Markup::M( $$text );
+	}
+
+	my $extras =
 		( 1 == @extras and (ref($extras[0])||'') eq 'HASH') ?
 	   	$extras[0] : # old style
 		{ @extras }; # new style
@@ -104,10 +103,12 @@ sub message_box
 	}
 
 	my $dlg = Prima::Dialog-> create(
+		borderStyle   => bs::Sizeable,
 		centered      => 1,
 		width         => 435,
 		height        => 125,
 		designScale   => [7, 18],
+		sizeMin       => [435, 125],
 		scaleChildren => 1,
 		visible       => 0,
 		text          => $title,
@@ -123,57 +124,75 @@ sub message_box
 	$dlg-> scaleChildren(0);
 
 	my $iconRight = 0;
-	my $iconView;
-	if ( $icon)
-	{
-		$icon = Prima::StdBitmap::icon( $icon);
-		if ( defined $icon) {
-			$iconView = $dlg-> insert( Widget =>
-				origin         => [ 
-					20, 
-					($dlg-> height + $fresh-> height - $icon-> height)/2
-				],
-				size           => [ $icon-> width, $icon-> height],
-				onPaint        => sub {
-					my $self = $_[1];
-					$self-> color( $dlg-> backColor);
-					$self-> bar( 0, 0, $self-> size);
-					$self-> put_image( 0, 0, $icon);
-				},
-			);
-
-			$iconRight = $iconView-> right;
-		}
+	if ( $icon and ($icon = Prima::StdBitmap::icon( $icon))) {
+		my $iconView = $dlg-> insert( Widget =>
+			growMode       => gm::GrowHiY,
+			left           => 20,
+			width          => $icon-> width,
+			top            => $dlg-> height - 10,
+			bottom         => $fresh-> height + 20,
+			ownerBackColor => 1,
+			onPaint        => sub {
+				my $self = $_[1];
+				$self-> clear;
+				$self-> put_image(0, ($self->height - $icon->height)/2, $icon);
+			},
+		);
+		$iconRight = $iconView-> right;
 	}
 
-	my $label = $dlg-> insert( Label =>
+	my $label = $dlg-> insert( 'Label' , text => $text );
+	unless ( defined $extras->{wordWrap} ) {
+		# can the text fit in a quarter of the screen?
+		my @as    = map { $_ / 2 } $::application->size;
+		my $gl    = int( $as[1] / $label-> font-> height);
+		my $lc    = scalar @{ $label-> text_wrap( $text, $as[0],
+			tw::NewLineBreak|tw::ReturnLines|tw::WordBreak|tw::ExpandTabs|tw::CalcTabs
+		)};
+		$extras->{wordWrap} = $lc <= $gl;
+	}
+
+	my %geom = (
 		left          => $iconRight + 15,
 		right         => $dlg-> width  - 10,
 		top           => $dlg-> height - 10,
 		bottom        => $fresh-> height + 20,
-		text          => $text,
-		autoWidth     => 0,
-		wordWrap      => 1,
-		showAccelChar => 1,
-		valignment    => ta::Middle,
 		growMode      => gm::Client,
 	);
+	my $scroller;
+	if ( $extras->{wordWrap}) {
+		$label->set(%geom,
+			autoWidth     => 0,
+			wordWrap      => 1,
+			growMode      => gm::Client,
+			valignment    => ta::Middle,
+		);
+	} else {
+		$scroller = $dlg-> insert( ScrollGroup => %geom);
+		$label->set(
+			owner         => $scroller->client,
+			growMode      => gm::Center,
+			autoWidth     => 1,
+			autoHeight    => 1,
+		);
+		$scroller->use_current_size;
+	}
 
-	my $gl = int( $label-> height / $label-> font-> height);
-	my $lc = scalar @{ $label-> text_wrap( 
-		$text, 
-		$label-> width, 
+	# expand the dialog
+	my $gl = int(( $dlg-> height - 10 - $fresh-> height - 20) / $label-> font-> height);
+	my $lc = scalar @{ $label-> text_wrap(
+		$text,
+		$label-> width,
 		tw::NewLineBreak|tw::ReturnLines|tw::WordBreak|tw::ExpandTabs|tw::CalcTabs
 	)};
 
-	if ( $lc > $gl) {
-		my $delta = ( $lc - $gl) * $label-> font-> height;
+	 if ( $lc > $gl) {
+		my $delta = ($lc - $gl) * $label-> font-> height;
 		my $dh = $dlg-> height;
-		$delta = $::application-> height - $dh 
-			if $dh + $delta > $::application-> height;
+		$delta = $::application-> height / 2 - $dh
+			if $dh + $delta > $::application-> height / 2;
 		$dlg-> height( $dh + $delta);
 		$dlg-> centered(1);
-		$iconView-> bottom( $iconView-> bottom + $delta / 2) if $iconView;
 	}
 
 	my $ret = $dlg-> execute;
@@ -191,8 +210,8 @@ sub input_box
 {
 	my ( $title, $text, $string, $buttons, @extras) = @_;
 	$buttons = mb::Ok|mb::Cancel unless defined $buttons;
-	
-	my $extras = 
+
+	my $extras =
 		( 1 == @extras and (ref($extras[0])||'') eq 'HASH') ?
 	   	$extras[0] : # old style
 		{ @extras }; # new style
@@ -217,7 +236,7 @@ sub input_box
 	);
 
 	my $input = $dlg-> insert( InputLine =>
-		size          => [ 415, 20],   
+		size          => [ 415, 20],
 		origin        => [ 10, 56],
 		text          => $string,
 		current       => 1,
@@ -226,8 +245,8 @@ sub input_box
 
 	my @ret = ( $dlg-> execute, $input-> text);
 	$dlg-> destroy;
-	return wantarray ? 
-		@ret : 
+	return wantarray ?
+		@ret :
 		(( $ret[0] == mb::OK || $ret[0] == mb::Yes) ? $ret[1] : undef);
 }
 
@@ -256,17 +275,19 @@ Prima::MsgBox - standard message and input dialog boxes
 
 The module contains two methods, C<message_box> and C<input_box>,
 that invoke correspondingly the standard message and one line text input
-dialog boxes. 
+dialog boxes.
 
 =head1 SYNOPSIS
 
-	use Prima;
-	use Prima::Application;
-	use Prima::MsgBox;
+	use Prima qw(Application);
+	use Prima::MsgBox qq(input_box message);
 
-	my $text = Prima::MsgBox::input_box( 'Sample input box', 'Enter text:', '');
-	$text = '(none)' unless defined $text;
-	Prima::MsgBox::message( "You have entered: '$text'", mb::Ok);
+	my $text = input_box( 'Sample input box', 'Enter text:', '') // '(none)';
+	message( \ "You have entered: 'B<Q<< $text >>>'", mb::Ok);
+
+=for podview <img src="msgbox.gif">
+
+=for html <p><img src="https://raw.githubusercontent.com/dk/Prima/master/pod/Prima/msgbox.gif">
 
 =head1 API
 
@@ -364,7 +385,7 @@ displayed.
 
 Only for C<input_box>.
 
-Contains a profile hash, passed to the input line as creation parameters. 
+Contains a profile hash, passed to the input line as creation parameters.
 
 =item buttons HASH
 
@@ -392,6 +413,18 @@ If set, the dialog owner is set to WINDOW, otherwise to C<$::main_window>.
 Necessary to maintain window stack order under some window managers, to disallow
 windows to be brought over the message box.
 
+=item wordWrap BOOLEAN=undef
+
+message_box can can display the message in two modes. In C<wordWrap = 1> where
+the text is expected to be relatively short, plus minus, several lines, where
+the user can resize the dialog if for some reason the text is too big.  In
+C<wordWrap = 0> mode there is added a scroller, so that even if the text indeed
+is too big to get on the screen event with the maximized dialog.
+
+By default the function analyzes the message text and decides which of the two
+modes is suited best. The explicit override is possible with this
+flag.
+
 =back
 
 =head1 AUTHOR
