@@ -289,8 +289,8 @@ extern void img_fill_dummy( PImage dummy, int w, int h, int type, Byte * data, R
 extern Bool img_put( Handle dest, Handle src, int dstX, int dstY, int srcX, int srcY, int dstW, int dstH, int srcW, int srcH, int rop, PBoxRegionRec region, Byte * color);
 extern Bool img_bar( Handle dest, int x, int y, int w, int h, PImgPaintContext ctx);
 extern void img_integral_rotate( Handle self, Byte * new_data, int new_line_size, int degrees);
-extern Bool img_generic_rotate( Handle self, float degrees, PImage dummy);
-extern Bool img_2d_transform( Handle self, float *matrix, PImage dummy);
+extern Bool img_generic_rotate( Handle self, float degrees, PImage output);
+extern Bool img_2d_transform( Handle self, float *matrix, PImage output);
 extern void img_mirror( Handle self, Bool vertically);
 extern Bool img_mirror_raw( int type, int w, int h, Byte * data, Bool vertically);
 extern void img_premultiply_alpha_constant( Handle self, int alpha);
